{
-------------------------------------------------------------------------------
Big Blue Mouse DataMerge Component
Copyright 2002 BigBlueMouse.com All rights reserved.
http://www.bigbluemouse.com/
--------------------------------------------------------------------------------
DEMONSTRATION VERSION ONLY!
This version of the component is for demonstration purposes only so that you can
determine if you want to purchase the full source code to the component. This
version has the most important routines contained in a separate run-time DLL.
The full version DOES NOT REQUIRE any supporting DLL files. The full version is
100% native Delphi source code and will compile right into your application.

TO PURCHASE THIS COMPONENT GO TO THIS URL:
  http://www.bigbluemouse.com/products/datamerge/purchase.html
--------------------------------------------------------------------------------
VERSION 1.00.021

Download the latest version from:
  http://www.bigbluemouse.com/products/datamerge/download.html
  
For technical support please visit our Frequently Asked Questions list at:
  http://www.bigbluemouse.com/products/datamerge/faq.html

Or send an e-mail message to:
  support@bigbluemouse.com
--------------------------------------------------------------------------------
You may not redistribute the source code for this component in any form. You may
not create a new component from this source code that does not add significant
new features to the ones provided by this component.

You may use this component in any private or commercial application without
paying a licensing fee if you have paid the one-time registration fee per
developer.
--------------------------------------------------------------------------------
}

unit bbmDataMerge;

interface

uses
  Windows, classes, sysutils;

type

  TLookupEvent = procedure (Sender: TObject; Source: string; var ReplaceWith: string) of object;
  TProgressEvent = procedure (Sender: TObject; Percent: integer) of object;

  EMissingFile = class(Exception);

  TBBMMergeSource = (msString, msFile, msStream);

  TBBMDataMerge = class;

  TBBMDataMergeVars = class
  protected
    FOwner: TBBMDataMerge;
    FVars: TStringList;
    FValues: TStringList;
    function GetCount: integer;
  public
    constructor Create(AOwner: TBBMDataMerge);
    destructor Destroy; override;
    procedure Add(VarName, Value: string);
    procedure Assign(Vars: TBBMDataMergeVars);
    procedure Delete(VarName: string);
    procedure Clear;
    function Value(VarName: string; var Value: string): boolean;
    property Count: integer read GetCount;
  end;

  TBBMDataMerge = class(TComponent)
  private
    FDelimEnd: string;
    FDelimStart: string;
    FMergeResult: TBBMMergeSource;
    FMergeSource: TBBMMergeSource;
    FOnLookup: TLookupEvent;
    FOnProgress: TProgressEvent;
    FResultFilename: string;
    FResultStream: TStream;
    FResultText: string;
    FSourceFilename: string;
    FSourceStream: TStream;
    FSourceText: string;
    FUpdateDLLVars: boolean;
    FVars: TBBMDataMergeVars;
    procedure SetResultFilename(v: string);
    procedure SetResultStream(v: TStream);
    procedure SetSourceFilename(v: string);
    procedure SetSourceStream(v: TStream);
    procedure SetSourceText(v: string);
  protected
    procedure VarsChanged;
  public
    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;
    procedure Merge;
    property MergeResult: TBBMMergeSource read FMergeResult write FMergeResult;
    property ResultFilename: string read FResultFilename write SetResultFilename;
    property ResultStream: TStream read FResultStream write SetResultStream;
    property ResultText: string read FResultText;
    property SourceFilename: string read FSourceFilename write SetSourceFilename;
    property SourceStream: TStream read FSourceStream write SetSourceStream;
    property SourceText: string read FSourceText write SetSourceText;
    property Vars: TBBMDataMergeVars read FVars;
  published
    property DelimEnd: string read FDelimEnd write FDelimEnd;
    property DelimStart: string read FDelimStart write FDelimStart;
    property OnLookup: TLookupEvent read FOnLookup write FOnLookup;
    property OnProgress: TProgressEvent read FOnProgress write FOnProgress;
  end;

procedure Register;

procedure MergeRT(FMergeSource: TBBMMergeSource; FMergeResult: TBBMMergeSource;
                  BufferDelimStart: PChar; BDSLength: integer;
                  BufferDelimEnd: PChar; BDELength: integer;
                  BufferSourceText: PChar; BSTLength: integer;
                  BufferSourceFilename: PChar; BSFLength: integer;
                  BufferResultFilename: PChar; BRFLength: integer;
                  SourceStream: TStream; ResultStream: TStream;
                  FObjHandle: integer;
                  ProgressCallback: FarProc; LookupCallback: FarProc;
                  ResultTextCallback: FarProc);
                  external 'bbmdmdrt';

procedure ClearVarsRT; external 'bbmdmdrt';

procedure SetVarsRT(BufferVarName: PChar; BVNLength: integer;
                    BufferVarValue: PChar; BVVLength: integer);
                    external 'bbmdmdrt';

procedure LookupAnswerRT(buffer: PChar; buffersize: integer);
                         external 'bbmdmdrt';

implementation

//The three callback procedures below are used by the DLL to communicate back to
//the demo component. The full version of the component does not use an external
//DLL and therefore does not require these callback procedures.

procedure ProgressCallback(ObjHandle, Percent: integer); stdcall;
var
  o: TBBMDataMerge;
begin
  o := TBBMDataMerge(ObjHandle);
  if assigned(o.FOnProgress) then o.FOnProgress(o, Percent);
end;

procedure LookupCallback(ObjHandle: integer; vbuffer: PChar; vbuffersize: integer); stdcall;
var
  o: TBBMDataMerge;
  varname, replacewith: string;
begin
  o := TBBMDataMerge(ObjHandle);
  if assigned(o.FOnLookup) then begin
    SetLength(varname, vbuffersize);
    move(vbuffer^, varname[1], vbuffersize);
    o.FOnLookup(o, varname, replacewith);
    LookupAnswerRT(PChar(replacewith), length(replacewith));
  end;
end;

procedure ResultTextCallback(ObjHandle: integer; vbuffer: PChar; vbuffersize: integer); stdcall;
var
  o: TBBMDataMerge;
  resulttext: string;
begin
  o := TBBMDataMerge(ObjHandle);
  SetLength(resulttext, vbuffersize);
  move(vbuffer^, resulttext[1], vbuffersize);
  o.FResultText := resulttext;
end;

//------------------------------------------------------------------------------
constructor TBBMDataMergeVars.Create(AOwner: TBBMDataMerge);
begin
  inherited Create;
  FOwner := AOwner;
  FVars := TStringList.Create;
  FValues := TStringList.Create;
end;

destructor TBBMDataMergeVars.Destroy;
begin
  FreeAndNil(FVars);
  FreeAndNil(FValues);
  inherited;
end;

procedure TBBMDataMergeVars.Assign(Vars: TBBMDataMergeVars);
begin
  FVars.Assign(Vars.FVars);
  FValues.Assign(Vars.FValues);
  if FOwner <> nil then FOwner.VarsChanged;
end;

procedure TBBMDataMergeVars.Clear;
begin
  FVars.Clear;
  FValues.Clear;
  if FOwner <> nil then FOwner.VarsChanged;
end;

function TBBMDataMergeVars.GetCount: integer;
begin
  result := FVars.Count;
end;

function TBBMDataMergeVars.Value(VarName: string; var Value: string): boolean;
var
  index: integer;
begin
  result := false;
  index := FVars.IndexOf(VarName);
  if index > -1 then begin
    result := true;
    Value := FValues[index];
  end;
end;

procedure TBBMDataMergeVars.Add(VarName, Value: string);
var
  index: integer;
begin
  index := FVars.IndexOf(VarName);
  if index > -1 then
    FValues[index] := Value
  else begin
    FVars.Add(VarName);
    FValues.Add(Value);
  end;
  if FOwner <> nil then FOwner.VarsChanged;
end;

procedure TBBMDataMergeVars.Delete(VarName: string);
var
  index: integer;
begin
  index := FVars.IndexOf(VarName);
  if index > -1 then begin
    FVars.Delete(index);
    FValues.Delete(index);
  end;
  if FOwner <> nil then FOwner.VarsChanged;
end;

//------------------------------------------------------------------------------
constructor TBBMDataMerge.Create(AOwner: TComponent);
begin
  inherited;
  FVars := TBBMDataMergeVars.Create(self);
  FMergeResult := msString;
  FUpdateDLLVars := false;
end;

destructor TBBMDataMerge.Destroy;
begin
  FreeAndNil(FVars);
  inherited;
end;

procedure TBBMDataMerge.Merge;
begin
  FUpdateDLLVars := true;
  VarsChanged;
  MergeRT(FMergeSource, FMergeResult,
          PChar(FDelimStart), Length(FDelimStart),
          PChar(FDelimEnd), Length(FDelimEnd),
          PChar(FSourceText), Length(FSourceText),
          PChar(FSourceFilename), Length(FSourceFilename),
          PChar(FResultFilename), Length(FResultFilename),
          FSourceStream, FResultStream, LParam(self), @ProgressCallback,
          @LookupCallback, @ResultTextCallback);
  ClearVarsRT;
  FUpdateDLLVars := false;
end;

procedure TBBMDataMerge.SetSourceText(v: string);
begin
  FSourceText := v;
  FMergeSource := msString;
end;

procedure TBBMDataMerge.SetSourceFilename(v: string);
begin
  FSourceFilename := v;
  FMergeSource := msFile;
end;

procedure TBBMDataMerge.SetSourceStream(v: TStream);
begin
  FSourceStream := v;
  FMergeSource := msStream;
end;

procedure TBBMDataMerge.SetResultFilename(v: string);
begin
  FResultFilename := v;
  FMergeResult := msFile;
end;

procedure TBBMDataMerge.SetResultStream(v: TStream);
begin
  FResultStream := v;
  FMergeResult := msStream;
end;

procedure TBBMDataMerge.VarsChanged;
var
  s1, s2: string;
  t: integer;
begin
  if FUpdateDLLVars then begin
    ClearVarsRT;
    if FVars.Count > 0 then
      for t := 0 to FVars.Count - 1 do begin
        s1 := FVars.FVars[t];
        s2 := FVars.FValues[t];
        SetVarsRT(pchar(s1), length(s1), pchar(s2), length(s2));
      end;
  end;
end;

//------------------------------------------------------------------------------
procedure Register;
begin
  RegisterComponents('Big Blue Mouse', [TBBMDataMerge]);
end;

end.
